import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

bool _isShowing = false;
BuildContext _context, _dismissingContext;

bool _isShowButtonClose = false;
double _borderRadius = 4.0;
Curve _insetAnimCurve = Curves.easeInOut;
Widget widget;

class DynamicDialog {
  _Dialog _dialog;

  DynamicDialog(BuildContext context,
      {Widget widgetInitial,bool isShowButton = false}) {
    assert(widgetInitial != null);
    _context = context;
    widget = widgetInitial;
    _isShowButtonClose = isShowButton ?? true;
  }

  void update({Widget widget,bool isShowButton}) {
    assert(widget != null);
    _isShowButtonClose = isShowButton?? _isShowButtonClose;
    _dialog.update(widget);
  }

  bool isShowing() {
    return _isShowing;
  }

  void dismiss() {
    if (_isShowing) {
      _isShowing = false;
      if (Navigator.of(_dismissingContext).canPop()) {
        Navigator.of(_dismissingContext).pop();
      } else {}
    }
  }

  Future<bool> hide() {
    if (_isShowing) {
      try {
        _isShowing = false;
        Navigator.of(_dismissingContext).pop(true);
        return Future.value(true);
      } catch (_) {
        return Future.value(false);
      }
    } else {
      return Future.value(false);
    }
  }

  void show() {
    if (!_isShowing) {
      _dialog = new _Dialog(widget);
      _isShowing = true;

      showDialog(
        context: _context,
        barrierDismissible: false,
        builder: (BuildContext context) {
          _dismissingContext = context;
          return WillPopScope(
            onWillPop: () async {
              Navigator.of(context).pop();
              return false;
            },
            child: Dialog(
                backgroundColor: Colors.transparent,
                insetAnimationCurve: _insetAnimCurve,
                insetAnimationDuration: Duration(milliseconds: 100),
                elevation: 0.0,
                shape: RoundedRectangleBorder(
                    borderRadius:
                        BorderRadius.all(Radius.circular(_borderRadius))),
                child: Scaffold(
                  body: Center(
                    child: _dialog,
                  ),
                  backgroundColor: Colors.transparent,
                  bottomNavigationBar: _isShowButtonClose ? Container(
                    height: 80,
                    child: Material(
                      color: Colors.black26,
                      type: MaterialType.circle,
                      child: IconButton(
                          icon: Container(
                            padding: EdgeInsets.all(8.0),
                            child: Icon(Icons.close),
                          ),
                          onPressed: () {
                            Navigator.of(context).pop();
                          }),
                    ),
                  ) : SizedBox(),
                )),
          );
        },
      );
    }
  }
}

// ignore: must_be_immutable
class _Dialog extends StatefulWidget {
  final Widget state;

  _Dialog(this.state);

  __DialogState ___dialogState = __DialogState();

  update(Widget state) {
    ___dialogState.update(state);
  }

  @override
  State<StatefulWidget> createState() {
    ___dialogState = __DialogState(state: state);
    return ___dialogState;
  }
}

class __DialogState extends State<_Dialog> {
  Widget state;

  __DialogState({this.state});

  update(Widget state) {
    setState(() {
      this.state = state;
    });
  }

  @override
  void dispose() {
    _isShowing = false;
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return state;
  }
}
